"""UserGroups module, all the logic to manage users from BeyondInsight API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_USERGROUPS_AD,
    POST_USERGROUPS_BI,
    POST_USERGROUPS_ENTRAID,
    POST_USERGROUPS_LDAP,
)
from secrets_safe_library.constants.users import UserType
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.usergroups import fields as usergroups_fields


class Usergroups(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger)

        # Schema rules used for validations
        self._schema = {
            "group_name": {"type": "string", "maxlength": 200, "nullable": True},
            "description": {"type": "string", "maxlength": 255, "nullable": True},
            "domain_name": {"type": "string", "maxlength": 250, "nullable": True},
            "group_distinguished_name": {
                "type": "string",
                "maxlength": 500,
                "nullable": True,
            },
            "membership_attribute": {
                "type": "string",
                "maxlength": 255,
                "nullable": True,
            },
            "account_attribute": {
                "type": "string",
                "maxlength": 255,
                "nullable": True,
            },
        }
        self._validator = Validator(self._schema)

    def get_usergroups(self) -> list:
        """
        Returns a list of active and inactive user groups.

        API: GET Usergroups

        Returns:
            list: List of Usergroups.
        """

        endpoint = "/usergroups"

        utils.print_log(
            self._logger,
            "Calling get_usergroups endpoint",
            logging.DEBUG,
        )

        response = self._run_get_request(endpoint, include_api_version=False)
        return response.json()

    def get_usergroup_by_id(self, usergroup_id: int) -> dict:
        """
        Returns a user group by ID.

        API: GET Usergroups/{usergroup_id}

        Args:
            usergroup_id (int): The Usergroup ID.

        Returns:
            dict: Usergroup.
        """

        endpoint = f"/usergroups/{usergroup_id}"

        utils.print_log(
            self._logger,
            "Calling get_usergroup_by_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_usergroups_by_name(self, name: str) -> list:
        """
        Returns a list of user groups by name.

        API: GET Usergroups?name={name}

        Args:
            name (str): The Usergroup name.

        Returns:
            list: List of Usergroups.
        """

        endpoint = f"/usergroups?name={name}"

        utils.print_log(
            self._logger,
            "Calling get_usergroups_by_name endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_usergroups_beyondinsight(
        self,
        group_name: str,
        description: str,
        is_active: bool = None,
        permissions: list = None,
        smart_rule_access: list = None,
        application_registration_ids: list = None,
    ) -> dict:
        """
        Creates a new user group, groupType = "BeyondInsight"

        API: POST Usergroups

        Args:
            group_name (str): The Usergroup name.
            description (str): The Usergroup description.
            is_active (bool, optional): Indicates if the Usergroup is active.
            permissions (list, optional): List of permissions.
                [ { PermissionID: int, AccessLevelID: int }, ... ]
            smart_rule_access (str, optional): List of smart rule access.
                [ { SmartRuleID: int, AccessLevelID: int }, ... ]
            application_registration_ids (list, optional): List of application
                                                           registration IDs.
                                                        [ int, … ]

        Returns:
            dict: Created Usergroup.
        """

        attributes = {
            "group_name": group_name,
            "description": description,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            usergroups_fields, POST_USERGROUPS_BI, Version.DEFAULT
        )

        # Convert string representation of dicts to actual dicts
        permissions = utils.convert_as_literal(permissions)
        smart_rule_access = utils.convert_as_literal(smart_rule_access)

        req_body = self.generate_request_body(
            req_structure,
            groupType=UserType.USER_TYPE_BI.value,
            group_name=group_name,
            description=description,
            is_active=is_active,
            permissions=permissions,
            smart_rule_access=smart_rule_access,
            application_registration_ids=application_registration_ids,
        )

        endpoint = "/usergroups"

        utils.print_log(
            self._logger,
            "Calling post_usergroups_beyondinsight endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def post_usergroups_entraid(
        self,
        description: str,
        group_name: str,
        client_id: int,
        tenant_id: int,
        client_secret: str,
        azure_instance: str = None,
        is_active: bool = None,
        permissions: list = None,
        smart_rule_access: list = None,
        application_registration_ids: list = None,
    ) -> dict:
        """
        Creates a new user group, groupType = "EntraId"

        API: POST Usergroups

        Args:
            description (str): The Usergroup description.
            group_name (str): The Usergroup name.
            client_id (int): The Entra ID.
            tenant_id (int): The Entra Tenant ID.
            client_secret (str): The Entra Client Secret.
            azure_instance (str, optional): The Entra Azure Instance.
            is_active (bool, optional): Indicates if the Usergroup is active.
            permissions (list, optional): List of permissions.
                [ { PermissionID: int, AccessLevelID: int }, ... ]
            smart_rule_access (str, optional): List of smart rule access.
                [ { SmartRuleID: int, AccessLevelID: int }, ... ]
            application_registration_ids (list, optional): List of application
                                                           registration IDs.
                                                        [ int, … ]

        Returns:
            dict: Created Usergroup.
        """

        attributes = {
            "description": description,
            "group_name": group_name,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            usergroups_fields, POST_USERGROUPS_ENTRAID, Version.DEFAULT
        )
        # Convert string representation of dicts to actual dicts
        permissions = utils.convert_as_literal(permissions)
        smart_rule_access = utils.convert_as_literal(smart_rule_access)

        req_body = self.generate_request_body(
            req_structure,
            groupType=UserType.USER_TYPE_ENTRAID.value,
            description=description,
            group_name=group_name,
            client_id=client_id,
            tenant_id=tenant_id,
            client_secret=client_secret,
            azure_instance=azure_instance,
            is_active=is_active,
            permissions=permissions,
            smart_rule_access=smart_rule_access,
            application_registration_ids=application_registration_ids,
        )

        endpoint = "/usergroups"

        utils.print_log(
            self._logger,
            "Calling post_usergroups_entraid endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def post_usergroups_ad(
        self,
        group_name: str,
        domain_name: str,
        description: str,
        forest_name: str = None,
        bind_user: str = None,
        bind_password: str = None,
        use_ssl: bool = None,
        is_active: bool = None,
        excluded_from_global_sync: bool = None,
        override_global_sync_settings: bool = None,
        permissions: list = None,
        smart_rule_access: list = None,
        application_registration_ids: list = None,
    ) -> dict:
        """
        Creates a new user group, groupType = "ActiveDirectory"

        API: POST Usergroups

        Args:
            group_name (str): The Usergroup name.
            domain_name (str): The Active Directory domain name.
            description (str): The Usergroup description.
            forest_name (str, optional): The Active Directory forest name.
            bind_user (str, optional): The Active Directory bind user.
            bind_password (str, optional): The Active Directory bind password.
            use_ssl (bool, optional): Use SSL.
            is_active (bool, optional): Indicates if the Usergroup is active.
            excluded_from_global_sync (bool, optional): Exclude from global sync.
            override_global_sync_settings (bool, optional): Override global sync
                                                            settings.
            permissions (list, optional): List of permissions.
                [ { PermissionID: int, AccessLevelID: int }, ... ]
            smart_rule_access (str, optional): List of smart rule access.
                [ { SmartRuleID: int, AccessLevelID: int }, ... ]
            application_registration_ids (list, optional): List of application
                                                        registration IDs.
                                                        [ int, … ]

        Returns:
            dict: Created Usergroup.
        """

        attributes = {
            "group_name": group_name,
            "domain_name": domain_name,
            "description": description,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            usergroups_fields, POST_USERGROUPS_AD, Version.DEFAULT
        )

        # Convert string representation of dicts to actual dicts
        permissions = utils.convert_as_literal(permissions)
        smart_rule_access = utils.convert_as_literal(smart_rule_access)

        req_body = self.generate_request_body(
            req_structure,
            groupType=UserType.USER_TYPE_AD.value,
            group_name=group_name,
            domain_name=domain_name,
            description=description,
            forest_name=forest_name,
            bind_user=bind_user,
            bind_password=bind_password,
            use_ssl=use_ssl,
            is_active=is_active,
            excluded_from_global_sync=excluded_from_global_sync,
            override_global_sync_settings=override_global_sync_settings,
            permissions=permissions,
            smart_rule_access=smart_rule_access,
            application_registration_ids=application_registration_ids,
        )

        endpoint = "/usergroups"

        utils.print_log(
            self._logger,
            "Calling post_usergroups_ad endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def post_usergroups_ldap(
        self,
        group_name: str,
        group_distinguished_name: str,
        host_name: str,
        membership_attribute: str,
        account_attribute: str,
        bind_user: str = None,
        bind_password: str = None,
        port: int = None,
        use_ssl: bool = None,
        is_active: bool = None,
        permissions: list = None,
        smart_rule_access: list = None,
        application_registration_ids: list = None,
    ) -> dict:
        """
        Creates a new user group, groupType = "LdapDirectory"

        API: POST Usergroups

        Args:
            group_name (str): The Usergroup name.
            group_distinguished_name (str): The LDAP group distinguished name.
            host_name (str): The LDAP host name.
            membership_attribute (str): The LDAP membership attribute.
            account_attribute (str): The LDAP account attribute.
            bind_user (str, optional): The LDAP bind user.
            bind_password (str, optional): The LDAP bind password.
            port (int, optional): The LDAP port.
            use_ssl (bool, optional): Use SSL.
            is_active (bool, optional): Indicates if the Usergroup is active.
            permissions (list, optional): List of permissions.
                [ { PermissionID: int, AccessLevelID: int }, ... ]
            smart_rule_access (str, optional): List of smart rule access.
                [ { SmartRuleID: int, AccessLevelID: int }, ... ]
            application_registration_ids (list, optional): List of application
                                                        registration IDs.
                                                        [ int, … ]

        Returns:
            dict: Created Usergroup.
        """

        attributes = {
            "group_name": group_name,
            "group_distinguished_name": group_distinguished_name,
            "membership_attribute": membership_attribute,
            "account_attribute": account_attribute,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            usergroups_fields, POST_USERGROUPS_LDAP, Version.DEFAULT
        )

        # Convert string representation of dicts to actual dicts
        permissions = utils.convert_as_literal(permissions)
        smart_rule_access = utils.convert_as_literal(smart_rule_access)

        req_body = self.generate_request_body(
            req_structure,
            groupType=UserType.USER_TYPE_LDAP.value,
            group_name=group_name,
            group_distinguished_name=group_distinguished_name,
            host_name=host_name,
            membership_attribute=membership_attribute,
            account_attribute=account_attribute,
            bind_user=bind_user,
            bind_password=bind_password,
            port=port,
            use_ssl=use_ssl,
            is_active=is_active,
            permissions=permissions,
            smart_rule_access=smart_rule_access,
            application_registration_ids=application_registration_ids,
        )

        endpoint = "/usergroups"

        utils.print_log(
            self._logger,
            "Calling post_usergroups_ldap endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def delete_usergroup_by_name(self, name: str) -> None:
        """
        Deletes a user group by name.

        API: DELETE Usergroups?name={name}

        Args:
            name (str): The Usergroup name.

        Returns:
            None
        """

        endpoint = f"/usergroups?name={name}"

        utils.print_log(
            self._logger,
            "Calling delete_usergroup_by_name endpoint",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)

        utils.print_log(
            self._logger,
            f"Usergroup with name {name} deleted successfully.",
            logging.INFO,
        )

    def delete_usergroup_by_id(self, usergroup_id: int) -> None:
        """
        Deletes a user group by ID.

        API: DELETE Usergroups/{usergroup_id}

        Args:
            usergroup_id (int): The Usergroup ID.

        Returns:
            None
        """

        endpoint = f"/usergroups/{usergroup_id}"

        utils.print_log(
            self._logger,
            "Calling delete_usergroup_by_id endpoint",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)

        utils.print_log(
            self._logger,
            f"Usergroup with ID {usergroup_id} deleted successfully.",
            logging.INFO,
        )



"""
This file contains the fields mapping for each endpoint and version related to
Usergroups.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_USERGROUPS,
    GET_USERGROUPS_ID,
    GET_USERGROUPS_NAME,
    POST_USERGROUPS_AD,
    POST_USERGROUPS_BI,
    POST_USERGROUPS_ENTRAID,
    POST_USERGROUPS_LDAP,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_USERGROUPS: {
        Version.DEFAULT: [
            "GroupID",
            "Name",
            "DistinguishedName",
            "Description",
            "GroupType",
            "AccountAttribute",
            "ApplicationRegistrationIDs",
            "MembershipAttribute",
            "IsActive",
        ],
    },
    GET_USERGROUPS_ID: {
        Version.DEFAULT: [
            "GroupID",
            "Name",
            "DistinguishedName",
            "Description",
            "GroupType",
            "AccountAttribute",
            "ApplicationRegistrationIDs",
            "MembershipAttribute",
            "IsActive",
        ],
    },
    GET_USERGROUPS_NAME: {
        Version.DEFAULT: [
            "GroupID",
            "Name",
            "DistinguishedName",
            "GroupType",
            "AccountAttribute",
            "ApplicationRegistrationIDs",
            "MembershipAttribute",
            "IsActive",
        ],
    },
    POST_USERGROUPS_BI: {
        Version.DEFAULT: {
            "groupType": str,
            "groupName": str,
            "description": str,
            "isActive": bool,
            "Permissions": [
                {
                    "PermissionID": int,
                    "AccessLevelID": int,
                },
            ],
            "SmartRuleAccess": [
                {"SmartRuleID": int, "AccessLevelID": int},
            ],
            "ApplicationRegistrationIDs": list[int],
        },
    },
    POST_USERGROUPS_ENTRAID: {
        Version.DEFAULT: {
            "groupType": str,
            "description": str,
            "groupName": str,
            "ClientId": str,
            "TenantId": str,
            "ClientSecret": str,
            "AzureInstance": str,
            "isActive": bool,
            "Permissions": [
                {
                    "PermissionID": int,
                    "AccessLevelID": int,
                },
            ],
            "SmartRuleAccess": [
                {"SmartRuleID": int, "AccessLevelID": int},
            ],
            "ApplicationRegistrationIDs": list[int],
        },
    },
    POST_USERGROUPS_AD: {
        Version.DEFAULT: {
            "groupType": str,
            "groupName": str,
            "forestName": str,
            "domainName": str,
            "description": str,
            "bindUser": str,
            "bindPassword": str,
            "useSSL": bool,
            "isActive": bool,
            "ExcludedFromGlobalSync": bool,
            "OverrideGlobalSyncSettings": bool,
            "Permissions": [
                {
                    "PermissionID": int,
                    "AccessLevelID": int,
                },
            ],
            "SmartRuleAccess": [
                {"SmartRuleID": int, "AccessLevelID": int},
            ],
            "ApplicationRegistrationIDs": list[int],
        },
    },
    POST_USERGROUPS_LDAP: {
        Version.DEFAULT: {
            "groupType": str,
            "groupName": str,
            "groupDistinguishedName": str,
            "hostName": str,
            "bindUser": str,
            "bindPassword": str,
            "port": int,
            "useSSL": bool,
            "membershipAttribute": str,
            "accountAttribute": str,
            "isActive": bool,
            "Permissions": [
                {
                    "PermissionID": int,
                    "AccessLevelID": int,
                },
            ],
            "SmartRuleAccess": [
                {"SmartRuleID": int, "AccessLevelID": int},
            ],
            "ApplicationRegistrationIDs": list[int],
        },
    },
}



"""Subscription Delivery module, all the logic to manage Subscription
Delivery from PS API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import ListMixin


class SubscriptionDelivery(APIObject, ListMixin):
    """Class to interact with Subscription Deliveries in PS API."""

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/subscriptions/delivery")

        # Schema rules used for validations
        self._schema = {
            "id": {"type": "integer", "nullable": False},
        }
        self._validator = Validator(self._schema)

    def download(self, request_id: int) -> list:
        """
        Downloads a specific subscription delivery by ID.

        API: POST Subscriptions/Delivery/download?id={id}

        Args:
            request_id (int): ID of the request for which to retrieve the subscription
                delivery.

        Returns:
            list: List of delivery details including ReportDeliveryId, ScheduleId,
                  Filename, ApplicationType, and Snapshot.
        """

        attributes = {"id": request_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"{self.endpoint}/download/id={request_id}"

        utils.print_log(
            self._logger,
            "Calling download_subscription_delivery endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint,
            payload={},
            include_api_version=False,
            expected_status_code=200,
        )

        return response.json()



"""Helper"""

# flake8: noqa: E402
# -*- coding: utf-8 -*-
# (c) 2025 BeyondTrust Inc.
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


def get_authentication_parameters():
    """
    Get common authentication parameters definition

    Returns:
        dict: Common parameters definition
    """
    params = dict(
        api_url=dict(type="str", required=True),
        client_id=dict(type="str", required=True, no_log=True),
        client_secret=dict(type="str", required=True, no_log=True),
        api_version=dict(type="str", required=True),
    )

    return params



"""Password rules module, all the logic to manage Password rules from PS API"""

import logging

from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import GetByIdMixin, ListByKeyMixin, ListMixin


class PasswordRule(APIObject, GetByIdMixin, ListMixin, ListByKeyMixin):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/passwordrules")



"""
This file contains the fields mapping for each endpoint and version related to
Access Levels.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_ACCESS_LEVELS,
    POST_ACCESS_LEVELS_USERGROUPID_SMARTRULEID,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_ACCESS_LEVELS: {
        Version.DEFAULT: [
            "AccessLevelID",
            "Name",
        ]
    },
    POST_ACCESS_LEVELS_USERGROUPID_SMARTRULEID: {
        Version.DEFAULT: {
            "AccessLevelID": int,
        }
    },
}



"""
This file contains the fields mapping for each endpoint and version related to Safes.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-api
- https://docs.beyondtrust.com/bips/docs/secrets-safe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_MANAGED_ACCOUNTS_ID,
    GET_MANAGED_ACCOUNTS_ID_ATTRIBUTES,
    GET_MANAGED_SYSTEMS_SYSTEM_ID_MANAGED_ACCOUNTS,
    POST_MANAGED_SYSTEMS_SYSTEM_ID_MANAGED_ACCOUNTS,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_MANAGED_SYSTEMS_SYSTEM_ID_MANAGED_ACCOUNTS: {
        Version.DEFAULT: [
            "ManagedAccountID",
            "ManagedSystemID",
            "DomainName",
            "AccountName",
            "DistinguishedName",
            "PasswordFallbackFlag",
            "UserPrincipalName",
            "SAMAccountName",
            "LoginAccountFlag",
            "Description",
            "PasswordRuleID",
            "ApiEnabled",
            "ReleaseNotificationEmail",
            "ChangeServicesFlag",
            "RestartServicesFlag",
            "ChangeTasksFlag",
            "ReleaseDuration",
            "MaxReleaseDuration",
            "ISAReleaseDuration",
            "MaxConcurrentRequests",
            "AutoManagementFlag",
            "DSSAutoManagementFlag",
            "CheckPasswordFlag",
            "ResetPasswordOnMismatchFlag",
            "ChangePasswordAfterAnyReleaseFlag",
            "ChangeFrequencyType",
            "ChangeFrequencyDays",
            "ChangeTime",
            "ParentAccountID",
            "IsSubscribedAccount",
            "LastChangeDate",
            "NextChangeDate",
            "IsChanging",
            "ChangeState",
            "UseOwnCredentials",
            "WorkgroupID",
            "ChangeIISAppPoolFlag",
            "RestartIISAppPoolFlag",
            "ChangeWindowsAutoLogonFlag",
            "ChangeComPlusFlag",
            "ChangeDComFlag",
            "ChangeSComFlag",
        ]
    },
    GET_MANAGED_ACCOUNTS_ID: {
        Version.DEFAULT: [
            "ManagedAccountID",
            "ManagedSystemID",
            "DomainName",
            "AccountName",
            "DistinguishedName",
            "PasswordFallbackFlag",
            "UserPrincipalName",
            "SAMAccountName",
            "LoginAccountFlag",
            "Description",
            "PasswordRuleID",
            "ApiEnabled",
            "ReleaseNotificationEmail",
            "ChangeServicesFlag",
            "RestartServicesFlag",
            "ChangeTasksFlag",
            "ReleaseDuration",
            "MaxReleaseDuration",
            "ISAReleaseDuration",
            "MaxConcurrentRequests",
            "AutoManagementFlag",
            "DSSAutoManagementFlag",
            "CheckPasswordFlag",
            "ResetPasswordOnMismatchFlag",
            "ChangePasswordAfterAnyReleaseFlag",
            "ChangeFrequencyType",
            "ChangeFrequencyDays",
            "ChangeTime",
            "ParentAccountID",
            "IsSubscribedAccount",
            "LastChangeDate",
            "NextChangeDate",
            "IsChanging",
            "ChangeState",
            "UseOwnCredentials",
            "WorkgroupID",
            "ChangeIISAppPoolFlag",
            "RestartIISAppPoolFlag",
            "ObjectID",
        ]
    },
    GET_MANAGED_ACCOUNTS_ID_ATTRIBUTES: {
        Version.DEFAULT: [
            "AttributeID",
            "AttributeTypeID",
            "ParentAttributeID",
            "ShortName",
            "LongName",
            "Description",
            "IsReadOnly",
        ]
    },
}

# POST fields according 3.0 to 3.5 versions (these have incremental changes)
POST_MANAGED_ACCOUNT_V3_0 = {
    "AccountName": str,
    "Password": str,
    "DomainName": str,
    "UserPrincipalName": str,
    "SAMAccountName": str,
    "DistinguishedName": str,
    "PrivateKey": str,
    "Passphrase": str,
    "PasswordFallbackFlag": bool,
    "LoginAccountFlag": bool,
    "Description": str,
    "PasswordRuleID": int,
    "ApiEnabled": bool,
    "ReleaseNotificationEmail": str,
    "ChangeServicesFlag": bool,
    "RestartServicesFlag": bool,
    "ChangeTasksFlag": bool,
    "ReleaseDuration": int,
    "MaxReleaseDuration": int,
    "ISAReleaseDuration": int,
    "MaxConcurrentRequests": int,
    "AutoManagementFlag": bool,
    "DSSAutoManagementFlag": bool,
    "CheckPasswordFlag": bool,
    "ResetPasswordOnMismatchFlag": bool,
    "ChangePasswordAfterAnyReleaseFlag": bool,
    "ChangeFrequencyType": str,
    "ChangeFrequencyDays": int,
    "ChangeTime": str,
    "NextChangeDate": str,
}

POST_MANAGED_ACCOUNT_V3_1 = {**POST_MANAGED_ACCOUNT_V3_0, "UseOwnCredentials": bool}
POST_MANAGED_ACCOUNT_V3_2 = {
    **POST_MANAGED_ACCOUNT_V3_1,
    "ChangeIISAppPoolFlag": bool,
    "RestartIISAppPoolFlag": bool,
}
POST_MANAGED_ACCOUNT_V3_3 = {**POST_MANAGED_ACCOUNT_V3_2, "WorkgroupID": int}
POST_MANAGED_ACCOUNT_V3_4 = {
    **POST_MANAGED_ACCOUNT_V3_3,
    "ChangeWindowsAutoLogonFlag": bool,
    "ChangeComPlusFlag": bool,
    "ChangeDComFlag": bool,
    "ChangeSComFlag": bool,
}
POST_MANAGED_ACCOUNT_V3_5 = {**POST_MANAGED_ACCOUNT_V3_4, "ObjectID": str}

post_fields = {
    POST_MANAGED_SYSTEMS_SYSTEM_ID_MANAGED_ACCOUNTS: {
        Version.V3_0.value: POST_MANAGED_ACCOUNT_V3_0,
        Version.V3_1.value: POST_MANAGED_ACCOUNT_V3_1,
        Version.V3_2.value: POST_MANAGED_ACCOUNT_V3_2,
        Version.V3_3.value: POST_MANAGED_ACCOUNT_V3_3,
        Version.V3_4.value: POST_MANAGED_ACCOUNT_V3_4,
        Version.V3_5.value: POST_MANAGED_ACCOUNT_V3_5,
    }
}

fields.update(post_fields)



from secrets_safe_library.validators.base import BaseValidator


class _FunctionalAccountValidator:
    common_schema = {}

    def get_create_schema(self):
        """
        Returns the schema for creating a functional account.
        """
        return {
            **self.common_schema,
            "platform_id": {"type": "integer", "nullable": False},
            "domain_name": {"type": "string", "maxlength": 500, "nullable": True},
            "account_name": {
                "type": "string",
                "maxlength": 245,
                "nullable": False,
            },
            "display_name": {
                "type": "string",
                "maxlength": 100,
                "nullable": True,
            },
            "password": {
                "type": "string",
                "maxlength": 1000,
                "nullable": True,
                "is_required_if": {
                    "field": "requires_secret",
                    "value": False,
                },
            },
            "private_key": {  # Comes from SSHConfig
                "type": "string",
                "nullable": True,
            },
            "passphrase": {  # Comes from SSHConfig
                "type": "string",
                "nullable": True,
            },
            "description": {
                "type": "string",
                "maxlength": 1000,
                "nullable": True,
            },
            "elevation_command": {  # Comes from SSHConfig
                "type": "string",
                "maxlength": 80,
                "nullable": True,
            },
            "tenant_id": {
                "type": "string",
                "maxlength": 36,
                "nullable": True,
            },
            "object_id": {
                "type": "string",
                "maxlength": 36,
                "nullable": True,
            },
            "secret": {
                "type": "string",
                "maxlength": 255,
                "nullable": True,
            },
            "service_account_email": {
                "type": "string",
                "maxlength": 255,
                "nullable": True,
            },
            "azure_instance": {
                "type": "string",
                "allowed": ["AzurePublic", "AzureUsGovernment"],
                "nullable": True,
            },
        }


class FunctionalAccountValidator(BaseValidator):
    """Validator for functional accounts operations."""

    def __init__(self):
        self.schema_validator = _FunctionalAccountValidator()

    def get_schema(self, operation: str) -> dict:
        """
        Retrieve the schema for the specified operation.

        Args:
            operation (str): The operation type (e.g., 'list', 'create').

        Returns:
            dict: The schema for the specified operation and version.
        """
        if operation == "create":
            return self.schema_validator.get_create_schema()

        raise ValueError(f"Unsupported operation: {operation}")

    def validate(
        self,
        data: dict,
        operation: str,
        allow_unknown: bool = True,
        update: bool = False,
    ) -> dict:
        schema = self.get_schema(operation)
        data = super().validate(data, schema, allow_unknown, update)
        return data



"""
Attribute Types module, all the logic to manage attribute types from BeyondInsight API
"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import DeleteByIdMixin, GetByIdMixin, ListMixin

# Maximum length for attribute type name
ATTRIBUTE_TYPE_NAME_MAX_LENGTH = 64


class AttributeType(APIObject, GetByIdMixin, DeleteByIdMixin, ListMixin):
    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/attributetypes")

        # Schema rules used for validations
        self._schema = {
            "name": {
                "type": "string",
                "maxlength": ATTRIBUTE_TYPE_NAME_MAX_LENGTH,
                "empty": False,
            },
        }

        self._validator = Validator(self._schema)

    def create_attribute_type(self, name: str) -> dict:
        """
        Creates a new Attribute Type.

        API: POST /AttributeTypes

        Args:
            name (str): The name of the attribute type.

        Returns:
            dict: Created Attribute Type object.
        """

        attributes = {"name": name}

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Name": name,
        }

        utils.print_log(
            self._logger,
            f"Calling create attribute type endpoint: {self.endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(
            self.endpoint, payload, include_api_version=False
        )

        return response.json()



"""Request Sets module, all the logic to manage requests from Password Safe API"""

import logging

from cerberus import Validator

from secrets_safe_library import utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import POST_REQUEST_SETS
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.request_sets import fields as requests_fields


class RequestSets(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/requestsets")

        # Schema rules used for validations
        self._schema = {
            "reason": {"type": "string", "maxlength": 1000, "nullable": True},
        }
        self._validator = Validator(self._schema)

    def get_request_sets(self, status: str = None) -> list:
        """
        Returns all Request Sets.

        API: GET RequestSets

        Args:
            status (str, optional): Status of requests to return. Options:
                    - all (default): Both active and pending requests.
                    - pending:  Requests that have not yet been approved.
                    - active: Requests approved (including auto-approved).

        Returns:
            list: List of Request Sets.
        """
        params = {"status": status}
        query_string = self.make_query_string(params)

        endpoint = f"{self.endpoint}?{query_string}"

        utils.print_log(
            self._logger,
            "Calling get_request_sets endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_request_sets(
        self,
        access_types: list,
        system_id: int,
        account_id: int,
        duration_minutes: int,
        application_id: int = None,
        reason: str = None,
        ticket_system_id: int = None,
        ticket_number: str = None,
    ) -> dict:
        """
        Creates a new Request Set.

        API: POST RequestSets

        Args:
            access_types (list): (at least two are required) A list of the types
                                of access requested (View, RDP, SSH, App).
            system_id (int): System ID.
            account_id (int): Account ID.
            duration_minutes (int, optional): Duration in minutes.
            application_id (int, optional): Application ID.
            reason (str, optional): Reason for the request.
            ticket_system_id (int, optional): Ticket system ID.
            ticket_number (str, optional): Ticket number.

        Returns:
            dict: Response from the API.
        """
        attributes = {
            "reason": reason,
        }

        if not self._validator.validate(attributes):
            raise utils.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            requests_fields, POST_REQUEST_SETS, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            access_types=access_types,
            system_id=system_id,
            account_id=account_id,
            duration_minutes=duration_minutes,
            application_id=application_id,
            reason=reason,
            ticket_system_id=ticket_system_id,
            ticket_number=ticket_number,
        )

        utils.print_log(
            self._logger,
            f"Calling post_request_sets endpoint: {self.endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(
            self.endpoint, req_body, include_api_version=False
        )

        return response.json()



"""Entity Types module, all the logic to manage entity types from PS API"""

import logging

from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import ListMixin


class EntityType(APIObject, ListMixin):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/entitytypes")



"""
Functional accounts module, all the logic to manage functional accounts from PS API
"""

import logging
from typing import Tuple

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import POST_FUNCTIONAL_ACCOUNTS
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.data_classes import SSHConfig
from secrets_safe_library.mapping.functional_accounts import (
    fields as functional_accounts_fields,
)
from secrets_safe_library.mixins import DeleteByIdMixin, GetByIdMixin, ListMixin
from secrets_safe_library.validators.functional_accounts import (
    FunctionalAccountValidator,
)


class FunctionalAccount(
    APIObject, GetByIdMixin, DeleteByIdMixin, ListMixin, FunctionalAccountValidator
):
    """
    FunctionalAccount class to manage functional accounts in Password Safe.
    """

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        APIObject.__init__(self, authentication, logger, endpoint="/functionalaccounts")

        FunctionalAccountValidator.__init__(self)

    def create_functional_account(
        self,
        *,
        platform_id: int,
        account_name: str,
        display_name: str = "",
        description: str = "",
        domain_name: str = "",
        password: str = "",
        ssh_config: SSHConfig = None,
        tenant_id: str = "",
        object_id: str = "",
        secret: str = "",
        service_account_email: str = "",
        azure_instance: str = "AzurePublic",
    ) -> Tuple[dict, int]:
        """
        Creates a new functional account.

        Args:
            platform_id (int): ID of the platform to which the account belongs.
            account_name (str): Name of the account.
            display_name (str, optional): Display name or alias for the account.
            description (str, optional): Description of the account.
            domain_name (str, optional): Domain name of the account.
            password (str, optional): Current account password.
            ssh_config (SSHConfig, optional): SSH configuration details if applicable.
            tenant_id (str, optional): Tenant ID if required by platform.
            object_id (str, optional): Object ID if required by platform.
            secret (str, optional): Secret if required by platform.
            service_account_email (str, optional): Service account email for Google
                accounts.
            azure_instance (str, optional): Azure instance type.

        Returns:
            Tuple[dict, int]: A tuple containing the created functional account object
                and the HTTP status code.
        """

        attributes = {
            "platform_id": platform_id,
            "account_name": account_name,
            "display_name": display_name,
            "description": description,
            "domain_name": domain_name,
            "password": password,
            "tenant_id": tenant_id,
            "object_id": object_id,
            "secret": secret,
            "service_account_email": service_account_email,
            "azure_instance": azure_instance,
        }

        if ssh_config and not isinstance(ssh_config, SSHConfig):
            raise exceptions.OptionsError(
                "ssh_config must be an instance of SSHConfig or None"
            )

        if ssh_config:
            attributes.update(ssh_config.__dict__)

        validated_data = self.validate(
            attributes, operation="create", allow_unknown=False
        )

        req_structure = self.get_request_body_version(
            functional_accounts_fields, POST_FUNCTIONAL_ACCOUNTS, Version.DEFAULT
        )

        req_body = self.generate_request_body(req_structure, **validated_data)
        utils.print_log(
            self._logger, "Calling create functional account endpoint", logging.DEBUG
        )
        response = self._run_post_request(
            endpoint=self.endpoint,
            payload=req_body,
            include_api_version=False,
        )
        return response.json(), response.status_code



"""Module Base class"""

# flake8: noqa: E402
# -*- coding: utf-8 -*-
# (c) 2025 BeyondTrust Inc.
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
import logging
from typing import Any

from ansible.module_utils.basic import AnsibleModule

from secrets_safe_library.integrations.ansible.common_utils.logger import Logger


class BaseModule:
    """
    Module Base class.
    """

    def __init__(self, module: AnsibleModule):
        self.module = module
        self.params = self.module.params
        self.res_args: dict[str, Any] = {}
        self.log = Logger(
            self.params.get("log_level", logging.DEBUG),
            self.params.get("log_file_name", "ansible-collection.log"),
        )

    def fail(self, msg: str):
        """
        Fail return.
        Args:
            msg (str): Error message.
        """
        self.module.fail_json(msg=msg, **self.res_args)

    def success(self, **kwargs):
        """
        Success return.
        Args:
            kwargs (dict): Data to return.
        """
        self.res_args.update(kwargs)
        self.module.exit_json(**self.res_args)



"""
This file contains the fields mapping for each endpoint and version related to Safes.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_PASSWORD_RULES,
    GET_PASSWORD_RULES_ENABLED_PRODUCTS,
    GET_PASSWORD_RULES_ID,
)
from secrets_safe_library.constants.versions import Version

GET_PASSWORD_RULES_FIELDS = [
    "PasswordRuleID",
    "Name",
    "Description",
    "MinimumLength",
    "MaximumLength",
    "FirstCharacterRequirement",
    "LowercaseRequirement",
    "UppercaseRequirement",
    "NumericRequirement",
    "SymbolRequirement",
    "ValidLowercaseCharacters",
    "ValidUppercaseCharacters",
    "ValidSymbols",
    "EnabledProducts",
]

fields = {
    GET_PASSWORD_RULES: {
        Version.DEFAULT: GET_PASSWORD_RULES_FIELDS,
    },
    GET_PASSWORD_RULES_ID: {
        Version.DEFAULT: GET_PASSWORD_RULES_FIELDS,
    },
    GET_PASSWORD_RULES_ENABLED_PRODUCTS: {
        Version.DEFAULT: GET_PASSWORD_RULES_FIELDS,
    },
}



"""Entitlements module, all the logic to manage entitlements from BeyondInsight API"""

import logging

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject


class Entitlement(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger)

    def list_entitlements(self, group_ids: list = None) -> list:
        """
        Returns user entitlements.
        Optionally filter by group IDs.

        API: GET Entitlements

        Args:
            group_ids (list, optional): List of group IDs to filter entitlements.
                If provided, only entitlements for these groups will be returned.

        Returns:
            list: List of user entitlements.
        """

        endpoint = "/entitlements"

        if group_ids:
            if not isinstance(group_ids, list):
                raise exceptions.OptionsError("group_ids must be a list.")
            group_ids_str = ",".join(map(str, group_ids))
            endpoint += f"?groupIds={group_ids_str}"

        utils.print_log(
            self._logger,
            "Calling list_entitlements endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint)

        return response.json()



"""
This file contains the fields mapping for each endpoint and version related to
Propagation actions.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/password-safe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_MANAGED_ACCOUNTS_PROPAGATION_ACTIONS,
    GET_PROPAGATION_ACTIONS,
    GET_PROPAGATION_ACTIONS_ID,
    POST_MANAGED_ACCOUNT_PROPAGATION_ACTIONS,
)
from secrets_safe_library.constants.versions import Version

DEFAULT_FIELDS = [
    "PropagationActionID",
    "PropagationActionTypeID",
    "Name",
    "Description",
]

fields = {
    GET_PROPAGATION_ACTIONS: {
        Version.DEFAULT: DEFAULT_FIELDS,
    },
    GET_PROPAGATION_ACTIONS_ID: {
        Version.DEFAULT: DEFAULT_FIELDS,
    },
    GET_MANAGED_ACCOUNTS_PROPAGATION_ACTIONS: {
        Version.DEFAULT: DEFAULT_FIELDS
        + [
            "SmartRuleID",
        ]
    },
    POST_MANAGED_ACCOUNT_PROPAGATION_ACTIONS: {
        Version.DEFAULT: {
            "SmartRuleID": int,
        }
    },
}



"""QuickRules module, all the logic to manage Quick Rules from PS API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import (
    DeleteByIdMixin,
    DeleteByKeyMixin,
    GetByIdMixin,
    ListByKeyMixin,
    ListMixin,
)


class QuickRule(
    APIObject,
    DeleteByIdMixin,
    DeleteByKeyMixin,
    GetByIdMixin,
    ListMixin,
    ListByKeyMixin,
):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/quickrules")

        # Schema rules used for validations
        self._schema = {
            "title": {"type": "string", "maxlength": 256, "nullable": True},
            "organization_id": {"type": "integer", "nullable": True},
            "category": {"type": "string", "maxlength": 256, "nullable": True},
            "description": {"type": "string", "maxlength": 512, "nullable": True},
            "rule_type": {
                "type": "string",
                "allowed": ["managedaccount", "managedsystem"],
                "nullable": True,
            },
        }
        self._validator = Validator(self._schema)

    def get_by_org_and_title(self, organization_id: int, title: str = None) -> dict:
        """
        Returns a Quick Rule by organization ID and title.

        API: GET Organizations/{orgID}/QuickRules?title={title}

        Args:
            organization_id (int): The Organization ID.
            title (str, optional): The QuickRule title.

        Returns:
            dict: QuickRule.
        """

        attributes = {"title": title}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        query_string = self.make_query_string(attributes)
        endpoint = f"/organizations/{organization_id}/quickrules?{query_string}"

        utils.print_log(
            self._logger,
            "Calling get_by_org_and_title endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def delete_by_org_and_title(self, organization_id: int, title: str) -> None:
        """
        Deletes a Quick Rule by organization ID and title.
        Only valid in a multi-tenant environment.

        API: DELETE Organizations/{orgID}/QuickRules?title={title}

        Args:
            organization_id (int): The Organization ID.
            title (str): The QuickRule title.

        Returns:
            None.
        """

        attributes = {"title": title}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        query_string = self.make_query_string(attributes)
        endpoint = f"/organizations/{organization_id}/quickrules?{query_string}"

        utils.print_log(
            self._logger,
            "Calling delete_by_org_and_title endpoint",
            logging.DEBUG,
        )
        _ = self._run_delete_request(endpoint)

    def create_quick_rule(
        self,
        ids: list[int],
        title: str,
        category: str = "Quick Rules",
        description: str = "",
        rule_type: str = "ManagedAccount",
    ) -> dict:
        """
        Creates a new Quick Rule with the managed accounts or systems referenced by ID
            and Rule Type.

        Args:
            ids (list): List of IDs to add to the Quick Rule.
            title (str): The title/name of the new Quick Rule.
            category (str, optional): The category in which to place the Quick Rule,
                default 'Quick Rules'.
            description (str, optional): The Quick Rule description.
            rule_type (str, optional): The type of the rule, either 'ManagedAccount' or
                'ManagedSystem', default 'ManagedAccount'.

        Returns:
            dict: The created Quick Rule.
        """
        payload = {
            "IDs": ids,
            "Title": title,
            "Category": category,
            "Description": description,
            "RuleType": rule_type,
        }

        utils.print_log(self._logger, "Creating Quick Rule with payload", logging.DEBUG)
        response = self._run_post_request(self.endpoint, payload=payload)

        return response.json()

    def add_accounts_to_quick_rule(
        self, quick_rule_id: int, account_ids: list[int]
    ) -> dict:
        """
        Updates the entire list of managed accounts in a Quick Rule by removing all
        Managed Account Fields - Quick Group ID filters and adding a new one with the
        managed accounts referenced by ID.

        API: PUT QuickRules/{quickRuleID}/ManagedAccounts

        Args:
            quick_rule_id (int): The ID of the Quick Rule.
            account_ids (list): List of account IDs to add to the Quick Rule.

        Returns:
            dict: The updated Quick Rule.
        """
        payload = {"AccountIDs": account_ids}

        utils.print_log(
            self._logger,
            f"Adding accounts to Quick Rule {quick_rule_id} with payload: {payload}",
            logging.DEBUG,
        )
        endpoint = f"{self.endpoint}/{quick_rule_id}/managedaccounts"
        response = self._run_put_request(endpoint, payload=payload)

        return response.json()



"""UserGroupRoles module, all the logic to manage user group roles from PS API"""

import logging
from typing import List

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import POST_USERGROUPS_ID_SMARTRULES_ROLES
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.user_group_roles import fields as roles_fields


class UserGroupRoles(APIObject):
    """
    Class to manage Password Safe roles for User Groups and Smart Rules.

    This class provides methods to get, set, and delete roles for specific
    User Group and Smart Rule combinations.
    """

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/usergroups")

        # Schema rules used for validations
        self._schema = {
            "roles": {
                "type": "list",
                "schema": {"type": "integer", "required": True},
                "required": False,
                "nullable": True,
            },
            "access_policy_id": {"type": "integer", "nullable": True},
        }
        self._validator = Validator(self._schema)

    def get_roles(self, user_group_id: int, smart_rule_id: int) -> List[dict]:
        """
        Returns a list of roles for the user group and Smart Rule referenced by ID.

        API: GET UserGroups/{userGroupId}/SmartRules/{smartRuleId}/Roles

        Args:
            user_group_id (int): ID of the user group.
            smart_rule_id (int): ID of the Smart Rule.

        Returns:
            List[dict]: List of roles with RoleID and Name.
        """
        endpoint = f"{self.endpoint}/{user_group_id}/smartrules/{smart_rule_id}/roles"

        utils.print_log(self._logger, "Calling get roles endpoint", logging.DEBUG)
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def set_roles(
        self,
        user_group_id: int,
        smart_rule_id: int,
        roles: List[int],
        access_policy_id: int = None,
    ) -> None:
        """
        Sets Password Safe roles for the user group and Smart Rule referenced by ID.

        API: POST UserGroups/{userGroupId}/SmartRules/{smartRuleId}/Roles

        Args:
            user_group_id (int): ID of the user group.
            smart_rule_id (int): ID of the Smart Rule.
            roles (List[int]): Zero or more roles to set on the UserGroup-SmartRule.
                Each role should be an integer representing the RoleID.
            access_policy_id (int, optional): The access policy ID to set on the
                UserGroup-SmartRule. Required when the Requestor or Requestor/Approver
                role is set.

        Returns:
            None.

        Raises:
            exceptions.OptionsError: If validation fails.
            exceptions.CreationError: If the API request fails.
        """

        # Validate input
        attributes = {"roles": roles}
        if access_policy_id is not None:
            attributes["access_policy_id"] = access_policy_id

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        # Convert roles to the required format for the API
        roles_dict = [{"RoleID": role_id} for role_id in roles]
        attributes["roles"] = roles_dict

        req_structure = self.get_request_body_version(
            roles_fields, POST_USERGROUPS_ID_SMARTRULES_ROLES, Version.DEFAULT
        )
        req_body = self.generate_request_body(
            req_structure,
            **attributes,
        )

        endpoint = f"{self.endpoint}/{user_group_id}/smartrules/{smart_rule_id}/roles"

        utils.print_log(self._logger, "Calling set roles endpoint", logging.DEBUG)
        self._run_post_request(
            endpoint, req_body, include_api_version=False, expected_status_code=204
        )

    def delete_roles(self, user_group_id: int, smart_rule_id: int) -> None:
        """
        Deletes all Password Safe roles for the user group and Smart Rule
        referenced by ID.

        API: DELETE UserGroups/{userGroupId}/SmartRules/{smartRuleId}/Roles

        Args:
            user_group_id (int): ID of the user group.
            smart_rule_id (int): ID of the Smart Rule.

        Returns:
            None.

        Raises:
            exceptions.DeletionError: If the API request fails.
        """
        endpoint = f"{self.endpoint}/{user_group_id}/smartrules/{smart_rule_id}/roles"

        utils.print_log(self._logger, "Calling delete roles endpoint", logging.DEBUG)
        self._run_delete_request(endpoint, expected_status_code=200)



"""
Address groups module, all the logic to manage address groups from BeyondInsight API
"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import DeleteByIdMixin, ListMixin


class AddressGroup(APIObject, DeleteByIdMixin, ListMixin):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/addressgroups")

        # Schema rules used for validations
        self._schema = {
            "name": {"type": "string", "maxlength": 256, "nullable": False},
        }
        self._validator = Validator(self._schema)

    def get_address_group_by_id(self, address_group_id: int) -> dict:
        """
        Find an address group by ID.

        API: GET AddressGroups/{id}

        Args:
            address_group_id (int): The address group ID.

        Returns:
            dict: Address group object according requested API version.
        """

        endpoint = f"{self.endpoint}/{address_group_id}"

        utils.print_log(
            self._logger,
            "Calling get_address_group_by_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_address_group_by_name(self, address_group_name: str) -> dict:
        """
        Returns an address group by name.

        API: GET AddressGroups/?name={name}

        Args:
            address_group_name (str): Name of the address group.

        Returns:
            dict: Address group.
        """

        endpoint = f"{self.endpoint}?name={address_group_name}"

        utils.print_log(
            self._logger,
            "Calling get_address_group_by_name endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def create_address_group(self, name: str) -> dict:
        """
        Creates a new address group.

        API: POST AddressGroups

        Args:
            name (str): Name of the address group.

        Returns:
            dict: Created address group object.
        """

        attributes = {"name": name}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Name": name,
        }

        utils.print_log(
            self._logger,
            f"Calling create_address_group endpoint: {self.endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(self.endpoint, payload)

        return response.json()

    def update_address_group(self, address_group_id: int, name: str) -> dict:
        """
        Updates an address group by ID.

        API: PUT AddressGroups/{id}

        Args:
            address_group_id (int): The address group ID.
            name (str): Name of the address group.

        Returns:
            dict: Updated address group object.
        """

        attributes = {"name": name}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Name": name,
        }

        endpoint = f"{self.endpoint}/{address_group_id}"

        utils.print_log(
            self._logger,
            "Calling update_address_group endpoint",
            logging.DEBUG,
        )
        response = self._run_put_request(endpoint, payload)

        return response.json()



"""
This file contains the fields mapping for each endpoint and version related to
Requests.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/password-safe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_REQUESTS,
    POST_REQUESTS,
    POST_REQUESTS_ALIASES,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_REQUESTS: {
        Version.DEFAULT: [
            "RequestID",
            "SystemID",
            "SystemName",
            "AccountID",
            "AccountName",
            "DomainName",
            "AliasID",
            "ApplicationID",
            "RequestReleaseDate",
            "ApprovedDate",
            "ExpiresDate",
            "Status",
            "AccessType",
            "Reason",
        ],
    },
    POST_REQUESTS: {
        Version.DEFAULT: {
            "AccessType": str,
            "SystemID": int,
            "AccountID": int,
            "ApplicationID": int,
            "DurationMinutes": int,
            "Reason": str,
            "AccessPolicyScheduleID": int,
            "ConflictOption": str,
            "TicketSystemID": int,
            "TicketNumber": str,
            "RotateOnCheckin": bool,
        },
    },
    POST_REQUESTS_ALIASES: {
        Version.DEFAULT: {
            "AccessType": str,
            "DurationMinutes": int,
            "Reason": str,
            "AccessPolicyScheduleID": int,
            "ConflictOption": str,
            "TicketSystemID": int,
            "TicketNumber": str,
            "RotateOnCheckin": bool,
        },
    },
}



"""Security module, includes sensitive data sanitization."""

import re


def sanitize_sensitive_data(message: str) -> str:
    """
    Sanitize sensitive information from log messages.

    Arguments:
        message (str): The original log message

    Returns:
        str: Sanitized log message with sensitive data redacted
    """

    if not isinstance(message, str):
        message = str(message)

    # Define sensitive data patterns

    # Compile regex patterns once at module initialization
    _COMPILED_PATTERNS = [
        (
            re.compile(
                r'("?api_?key"?\s*[:=]\s*["\']?)([a-zA-Z0-9+/=_-]{20,200})(["\']?)'
            ),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(r'("?key"?\s*[:=]\s*["\']?)([a-zA-Z0-9+/=_-]{20,200})(["\']?)'),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(r'("?password"?\s*[:=]\s*["\'])([^"\']+)(["\'])'),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(r'("?password"?\s*[:=]\s*)([^\s;&,"\']+)(?=[\s;&,]|$)'),
            r"\1***REDACTED***",
        ),
        (
            re.compile(r'("?pwd"?\s*[:=]\s*["\'])([^"\']+)(["\'])'),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(r'("?pwd"?\s*[:=]\s*)([^\s;&,"\']+)(?=[\s;&,]|$)'),
            r"\1***REDACTED***",
        ),
        (
            re.compile(r'("?passwd"?\s*[:=]\s*["\'])([^"\']+)(["\'])'),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(r'("?passwd"?\s*[:=]\s*)([^\s;&,"\']+)(?=[\s;&,]|$)'),
            r"\1***REDACTED***",
        ),
        (
            re.compile(
                r'("?client_?secret"?\s*[:=]\s*["\']?)'
                r'([a-zA-Z0-9+/=_-]{20,200})(["\']?)'
            ),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(
                r'("?secret"?\s*[:=]\s*["\']?)([a-zA-Z0-9+/=_-]{20,200})(["\']?)'
            ),
            r"\1***REDACTED***\3",
        ),
        (re.compile(r"(Bearer\s+)([a-zA-Z0-9+/=._-]{20,500})"), r"\1***REDACTED***"),
        (
            re.compile(
                r'("?token"?\s*[:=]\s*["\']?)([a-zA-Z0-9+/=._-]{20,500})(["\']?)'
            ),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(
                r'("?access_?token"?\s*[:=]\s*["\']?)'
                r'([a-zA-Z0-9+/=._-]{20,500})(["\']?)'
            ),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(
                r'(Authorization["\']?\s*[:=]\s*["\']?)([^"\'\\n\\r]{10,500})(["\']?)'
            ),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(r"(-----BEGIN[^-]*-----)(.*?)(-----END[^-]*-----)", re.DOTALL),
            r"\1***REDACTED***\3",
        ),
        (
            re.compile(r"(https?://)([^:/@]+):([^@]+)(@[^\s]+)"),
            r"\1\2:***REDACTED***\4",
        ),
        (re.compile(r"(eyJ[a-zA-Z0-9+/=._-]{20,})"), r"***REDACTED_JWT***"),
        (re.compile(r'(["\'][a-zA-Z0-9+/=]{40,}["\'])'), r"***REDACTED***"),
    ]

    # Apply sanitization patterns
    sanitized_message = message
    for cregex, replacement in _COMPILED_PATTERNS:
        sanitized_message = cregex.sub(replacement, sanitized_message)

    # Additional keyword-based sanitization for common sensitive field names
    # Note: Skip keywords already handled by main patterns to avoid double-processing
    sensitive_keywords = [
        "password",
        "passwd",
        "pwd",
        "secret",
        "token",
        "key",
        "api_key",
        "apikey",
        "client_secret",
        "client_key",
        "private_key",
        "certificate",
        "cert",
        "credential",
        "auth",
        "authorization",
        "access_token",
        "refresh_token",
    ]

    # Look for patterns like "field": "value" or field=value where field is sensitive
    for keyword in sensitive_keywords:
        # JSON-style assignments
        pattern = rf'("{keyword}"\s*:\s*")([^"]+)(")'
        sanitized_message = re.sub(
            pattern, r"\1***REDACTED***\3", sanitized_message, flags=re.IGNORECASE
        )

        # Form-style assignments for unquoted values
        # Use word boundaries and more specific delimiters
        pattern = rf'\b({keyword}\s*=\s*)([^\s;&,"\']+)(?=[\s;&,]|$)'
        sanitized_message = re.sub(
            pattern, r"\1***REDACTED***", sanitized_message, flags=re.IGNORECASE
        )

        # Form-style assignments for quoted values (single and double quotes)
        pattern = rf'\b({keyword}\s*=\s*")([^"]*?)(")'
        sanitized_message = re.sub(
            pattern, r"\1***REDACTED***\3", sanitized_message, flags=re.IGNORECASE
        )

        pattern = rf"\b({keyword}\s*=\s*')([^']*?)(')"
        sanitized_message = re.sub(
            pattern, r"\1***REDACTED***\3", sanitized_message, flags=re.IGNORECASE
        )

    return sanitized_message



"""Logger class, used for logging messages."""

# flake8: noqa: E402
# -*- coding: utf-8 -*-
# (c) 2025 BeyondTrust Inc.
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
try:
    import logging
    from logging.handlers import RotatingFileHandler
except ImportError:
    pass


class Logger:
    """
    Logger class, used to define a logger instance.
    """

    def __init__(self, log_level: int, log_file_name: str) -> None:
        """
        Initialize a new Logger instance.

        Args:
            log_level (int): Logging level.
            log_file_name (str): Name of the log file.
        """
        self.logger = logging.getLogger(log_file_name)
        self.logger.setLevel(log_level)

        if not self.logger.handlers:
            handler = RotatingFileHandler(
                f"{log_file_name}.log",
                maxBytes=1_000_000,
                backupCount=5,
                encoding="utf-8",
            )
            formatter = logging.Formatter(
                "%(asctime)-5s %(name)-15s %(levelname)-8s %(message)s"
            )
            handler.setFormatter(formatter)
            self.logger.addHandler(handler)

        return



from secrets_safe_library.constants.versions import Version
from secrets_safe_library.validators.base import BaseValidator


class _APIRegistrationSchemaValidator:
    def __init__(self):
        self.common_schema = {
            "access_token_duration": {"type": "integer", "min": 1, "nullable": True},
            "active": {"type": "boolean", "default": True},
            "visible": {"type": "boolean", "default": True},
            "multi_factor_authentication_enforced": {
                "type": "boolean",
                "default": False,
            },
            "client_certificate_required": {"type": "boolean", "default": False},
            "user_password_required": {"type": "boolean", "default": False},
            "verify_psrun_signature": {"type": "boolean", "default": False},
            "ip_authentication_rules": {
                "type": "list",
                "schema": {
                    "type": "dict",
                    "schema": {
                        "Type": {"type": "string", "nullable": False},
                        "Value": {"type": "string", "nullable": False},
                        "Description": {"type": "string", "nullable": True},
                    },
                },
                "nullable": True,
            },
            "psrun_rules": {
                "type": "list",
                "schema": {
                    "type": "dict",
                    "schema": {
                        "Id": {"type": "integer", "nullable": True},
                        "IPAddress": {"type": "string", "nullable": True},
                        "MacAddress": {"type": "string", "nullable": True},
                        "SystemName": {"type": "string", "nullable": True},
                        "FQDN": {"type": "string", "nullable": True},
                        "DomainName": {"type": "string", "nullable": True},
                        "UserId": {"type": "string", "nullable": True},
                        "RootVolumeId": {"type": "string", "nullable": True},
                        "OSVersion": {"type": "string", "nullable": True},
                    },
                },
                "nullable": True,
            },
            "x_forwarded_for_authentication_rules": {
                "type": "list",
                "schema": {
                    "type": "dict",
                    "schema": {
                        "Type": {"type": "string", "nullable": False},
                        "Value": {"type": "string", "nullable": False},
                        "Description": {"type": "string", "nullable": True},
                    },
                },
                "nullable": True,
            },
        }

    def get_create_schema(self, version: str = Version.DEFAULT.value) -> dict:
        """
        Get schema for creating API registrations.

        Args:
            version (str): The version (e.g., '3.0', '3.1', '3.2', '3.3', 'DEFAULT')

        Returns:
            dict: The schema for create operation.
        """
        schema = self.common_schema.copy()
        schema["name"] = {"type": "string", "maxlength": 256, "nullable": False}
        schema["registration_type"] = {
            "type": "string",
            "allowed": ["apikeypolicy", "apiaccesspolicy"],
            "nullable": False,
        }
        return schema

    def get_update_schema(self, version: str = Version.DEFAULT.value) -> dict:
        """
        Get schema for updating API registrations.

        Args:
            version (str): The version (e.g., '3.0', '3.1', '3.2', '3.3', 'DEFAULT')

        Returns:
            dict: The schema for update operation.
        """
        schema = self.common_schema.copy()
        schema["registration_id"] = {"type": "integer", "nullable": False}
        schema["name"] = {"type": "string", "maxlength": 256, "nullable": True}
        schema["registration_type"] = {
            "type": "string",
            "allowed": ["apikeypolicy", "apiaccesspolicy"],
            "nullable": True,
        }
        return schema

    def get_get_key_schema(self, version: str = Version.DEFAULT.value) -> dict:
        """
        Get schema for getting API key by ID.

        Args:
            version (str): The version (e.g., '3.0', '3.1', '3.2', '3.3', 'DEFAULT')

        Returns:
            dict: The schema for get key operation.
        """
        return {"api_registration_id": {"type": "integer", "nullable": False}}

    def get_rotate_key_schema(self, version: str = Version.DEFAULT.value) -> dict:
        """
        Get schema for rotating API key.

        Args:
            version (str): The version (e.g., '3.0', '3.1', '3.2', '3.3', 'DEFAULT')

        Returns:
            dict: The schema for rotate key operation.
        """
        return {"api_registration_id": {"type": "integer", "nullable": False}}


class APIRegistrationValidator(BaseValidator):
    """Validator for API registration operations."""

    def __init__(self):
        self.schema_validator = _APIRegistrationSchemaValidator()

    def get_schema(self, operation: str, version: str = Version.DEFAULT.value) -> dict:
        """
        Retrieve the schema for the specified operation and version.

        Args:
            operation (str): The operation type (e.g., 'create', 'update',
                'get_key', 'rotate_key').
            version (str): The version (e.g., '3.0', '3.1', '3.2', '3.3', 'DEFAULT')

        Returns:
            dict: The schema for the specified operation and version.
        """
        if operation == "create":
            return self.schema_validator.get_create_schema(version)

        if operation == "update":
            return self.schema_validator.get_update_schema(version)

        if operation == "get_key":
            return self.schema_validator.get_get_key_schema(version)

        if operation == "rotate_key":
            return self.schema_validator.get_rotate_key_schema(version)

        raise ValueError(f"Unsupported operation: {operation}")

    def validate(
        self,
        data: dict,
        operation: str,
        version: str = Version.DEFAULT.value,
        allow_unknown: bool = True,
        update: bool = False,
    ) -> dict:
        """
        Validate data for API registration operations.

        Args:
            data (dict): Data to validate.
            operation (str): The operation type (e.g., 'create', 'update',
                'get_key', 'rotate_key').
            version (str): The version (e.g., '3.0', '3.1', '3.2', '3.3', 'DEFAULT')
            allow_unknown (bool): Whether to allow unknown fields in the data.
            update (bool): Whether the validation is for an update operation.

        Returns:
            dict: The validated data.
        """
        schema = self.get_schema(operation, version)
        data = super().validate(data, schema, allow_unknown, update)
        return data



"""
DSS Key Policies module, all the logic to manage DSS Key Policies from Password Safe API
"""

import logging

from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import GetByIdMixin, ListMixin


class DSSKeyPolicies(APIObject, ListMixin, GetByIdMixin):
    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/dsskeyrules")



"""CertUtil Module"""

import logging
from pathlib import Path

from cryptography.hazmat.primitives.serialization import (
    Encoding,
    NoEncryption,
    PrivateFormat,
)
from cryptography.hazmat.primitives.serialization.pkcs12 import (
    load_key_and_certificates,
)

from secrets_safe_library import exceptions, utils


class CertUtil:

    _logger = None

    # Check the scope
    certificate = None
    certificate_key = None

    def __init__(self, certificate_path, certificate_password, logger=None):

        self._logger = logger
        self.set_certificate_data_from_pfx_file(certificate_path, certificate_password)

    def get_certificate(self):
        """
        Get certificate
        Arguments:
        Returns:
            certificate
        """
        if self.certificate:
            return str(self.certificate, encoding="utf-8")
        return None

    def get_certificate_key(self):
        """
        Get certificate key
        Arguments:
        Returns:
            certificate key
        """
        if self.certificate_key:
            return str(self.certificate_key, encoding="utf-8")
        return None

    def set_certificate_data_from_pfx_file(
        self, certificate_path, certificate_password
    ):
        """
        Decrypt .pfx file, get certificate key and cert content and set them to class
        attributes
        Arguments:
            certificate_path
            certificate_password
        Returns:
        """

        if not certificate_path:
            utils.print_log(
                self._logger, "Certificate was not configured", logging.INFO
            )
            self.certificate = None
            self.certificate_key = None
            return None

        try:
            self.certificate, self.certificate_key = (
                self.get_certificate_and_certificate_key(
                    certificate_path, certificate_password
                )
            )

        except FileNotFoundError as e:
            raise FileNotFoundError(f"Certificate not found: {e}")
        except Exception as e:
            raise exceptions.AuthenticationFailure(
                f"Missing certificate password or incorrect certificate password: {e}"
            )

    def get_certificate_and_certificate_key(
        self, certificate_path, certificate_password
    ):
        """
        Get certificate and certificate key from pfx file
        Arguments:
            certificate_path
            certificate_password
        Returns:
        """
        pfx = Path(certificate_path).read_bytes()
        private_key, main_cert, _ = load_key_and_certificates(
            pfx, certificate_password.encode("utf-8"), None
        )
        return main_cert.public_bytes(Encoding.PEM), private_key.private_bytes(
            Encoding.PEM, PrivateFormat.PKCS8, NoEncryption()
        )



"""Workgroups module, all the logic to manage workgroups from BeyondInsight API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import POST_WORKGROUPS
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.workgroups import fields as workgroups_fields


class Workgroup(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger)

        # Schema rules used for validations
        self._schema = {
            "name": {"type": "string", "maxlength": 256, "nullable": True},
            "workgroup_id": {"type": "integer", "nullable": True},
        }
        self._validator = Validator(self._schema)

    def get_workgroup_by_id(self, workgroup_id: int) -> dict:
        """
        Returns a Workgroup by ID.

        API: GET Workgroups/{id}

        Args:
            workgroup_id (int): The Workgroup ID.

        Returns:
            dict: Workgroup.
        """

        attributes = {"workgroup_id": workgroup_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/workgroups/{workgroup_id}"

        utils.print_log(
            self._logger,
            "Calling get_workgroup_by_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_workgroup_by_name(self, workgroup_name: str) -> dict:
        """
        Returns a Workgroup by name.

        API: GET Workgroups?name={name}

        Args:
            workgroup_name (str): The Workgroup name.

        Returns:
            dict: Workgroup.
        """

        attributes = {"name": workgroup_name}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/workgroups?name={workgroup_name}"

        utils.print_log(
            self._logger,
            "Calling get_workgroup_by_name endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_workgroups(self) -> list:
        """
        Returns all Workgroups.

        API: GET Workgroups

        Returns:
            list: List of Workgroups.
        """

        endpoint = "/workgroups"

        utils.print_log(
            self._logger,
            "Calling get_workgroups endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_workgroup(self, name: str, organization_id: int) -> dict:
        """
        Creates a new Workgroup.

        API: POST Workgroups

        Args:
            name (str): The Workgroup name.
            organization_id (int, optional): The Organization ID.

        Returns:
            dict: Created Workgroup.
        """

        attributes = {"name": name}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            workgroups_fields, POST_WORKGROUPS, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            organization_id=organization_id,
            name=name,
        )

        endpoint = "/workgroups"

        utils.print_log(
            self._logger,
            "Calling post_workgroup endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()



"""Aliases module, all the logic to manage aliases from PS API"""

import logging
from typing import List

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import GetByIdMixin, ListByKeyMixin


class Aliases(APIObject, ListByKeyMixin, GetByIdMixin):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/aliases")

        # Schema rules used for validations
        self._schema = {
            "state": {
                "type": ["integer", "list"],
                "nullable": True,
                "default": [1, 2],
                "schema": {"type": "integer", "allowed": [0, 1, 2]},
                "allowed": [0, 1, 2],
            }
        }
        self._validator = Validator(self._schema)

    def get_aliases(self, state: int | List[int] = [1, 2]):
        """
        Retrieve a list of aliases.

        Args:
            state (int | List[int], optional): Zero or more state values. (Default: 1,2)
                                   i.e., 'state=2', 'state=1,2', 'state=0,1,2'.
                        - 0: Unmapped
                        - 1: Mapped
                        - 2: Highly Available

        Returns:
            list: A list of aliases.
        """

        params = {"state": state}

        if not self._validator.validate(params):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        if isinstance(state, list):
            state = ",".join(map(str, state))

        endpoint = f"{self.endpoint}?state={state}"

        utils.print_log(self._logger, "Calling get_aliases endpoint", logging.DEBUG)

        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()



"""Organizations module, all the logic to manage organizations from BeyondInsight API"""

import logging

from secrets_safe_library import utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject


class Organization(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger)

    def get_organization_by_id(self, organization_id: str) -> dict:
        """
        Find an organization by ID.

        API: GET Organizations/{id}

        Args:
            organization_id (str): The organization ID (GUID).

        Returns:
            dict: Organization object.
        """

        endpoint = f"/organizations/{organization_id}"

        utils.print_log(
            self._logger,
            "Calling get_organization_by_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def list_organizations(self) -> list:
        """
        Returns a list of organizations to which the current user has permission.

        API: GET Organizations

        Returns:
            list: List of organizations.
        """

        endpoint = "/organizations"

        utils.print_log(
            self._logger,
            "Calling list_organizations endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_organization_by_name(self, organization_name: str) -> dict:
        """
        Returns an Organization by name.

        API: GET Organizations?name={name}

        Args:
            organization_name (str): Name of the organization.

        Returns:
            dict: Organization.
        """

        endpoint = f"/organizations?name={organization_name}"

        utils.print_log(
            self._logger,
            "Calling get_organization_by_name endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()



"""
This file contains the fields mapping for each endpoint and version related to
Ticket Systems.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/password-safe-apis
"""

from secrets_safe_library.constants.endpoints import GET_TICKET_SYSTEMS
from secrets_safe_library.constants.versions import Version

fields = {
    GET_TICKET_SYSTEMS: {
        Version.DEFAULT: [
            "TicketSystemID",
            "IsActive",
            "TicketSystemName",
            "Description",
            "IsDefaultSystem",
        ],
    },
}



# flake8: noqa: S105
"""
Endpoints list used in Integration Library (verb:endpoint)
"""

# Secrets
GET_SECRETS_SAFE_SECRETS = "get:secrets-safe/secrets"
GET_SECRETS_SAFE_SECRETS_SECRETID = "get:secrets-safe/secrets/{secretid}"
GET_SECRETS_SAFE_SECRETS_SECRETID_TEXT = "get:secrets-safe/secrets/{secretid}/text"
GET_SECRETS_SAFE_SECRETS_SECRETID_FILE = "get:secrets-safe/secrets/{secretid}/file"
GET_SECRETS_SAFE_FOLDERS_FOLDERID_SECRETS = (
    "get:secrets-safe/folders/{folderid}/secrets"
)
GET_SECRETS_SAFE_FOLDERS_FOLDERID_TEXT = "get:secrets-safe/folders/{folderid}/text"
GET_SECRETS_SAFE_FOLDERS_FOLDERID_FILE = "get:secrets-safe/folders/{folderid}/file"
POST_SECRETS_SAFE_FOLDERS_FOLDERID = "post:secrets-safe/folders/{folderid}/secrets"
POST_SECRETS_SAFE_FOLDERS_FOLDERID_TEXT = (
    "post:secrets-safe/folders/{folderid}/secrets/text"
)
POST_SECRETS_SAFE_FOLDERS_FOLDERID_FILE = (
    "post:secrets-safe/folders/{folderid}/secrets/file"
)
PUT_SECRETS_SAFE_SECRETS_SECRETID = "put:secrets-safe/secrets/{secretid}"
PUT_SECRETS_SAFE_SECRETS_SECRETID_TEXT = "put:secrets-safe/secrets/{secretid}/text"
PUT_SECRETS_SAFE_SECRETS_SECRETID_FILE = "put:secrets-safe/secrets/{secretid}/file"

# Folders
GET_SECRETS_SAFE_FOLDERS = "get:secrets-safe/folders"
GET_SECRETS_SAFE_FOLDERS_FOLDERID = "get:secrets-safe/folders/{folderid}"

# Managed Systems
GET_MANAGED_SYSTEMS = "get:managedsystems"
GET_MANAGED_SYSTEMS_MANAGEDSYSTEMID = "get:managedsystems/{managedsystemid}"
GET_MANAGED_SYSTEMS_ASSETID = "get:assets/{assetid}/managedsystems"
GET_MANAGED_SYSTEMS_DATABASEID = "get:databases/{databaseid}/managedsystems"
GET_MANAGED_SYSTEMS_FUNCTIONALACCOUNTID = (
    "get:functionalaccounts/{functionalaccountid}/managedsystems"
)
GET_MANAGED_SYSTEMS_WORKGROUPID = "get:workgroups/{workgroupid}/managedsystems"
POST_MANAGED_SYSTEMS_ASSETID = "post:assets/{assetid}/managedsystems"
POST_MANAGED_SYSTEMS_DATABASEID = "post:databases/{databaseid}/managedsystems"
POST_MANAGED_SYSTEMS_WORKGROUPID = "post:workgroups/{workgroupid}/managedsystems"
DELETE_MANAGED_SYSTEMS_MANAGEDSYSTEMID = "delete:managedsystems/{managedsystemid}"
PUT_MANAGED_SYSTEMS_MANAGEDSYSTEMID = "put:managedsystems/{managedsystemid}"

# Safes
GET_SECRETS_SAFE_SAFES = "get:secrets-safe/safes"
GET_SECRETS_SAFE_SAFES_ID = "get:secrets-safe/safes/{ID}"

# Assets
GET_WORKGROUPS_ID_ASSETS = "get:workgroups/{workgroupid}/assets"
GET_ASSETS_ID_ATTRIBUTES = "get:assets/{assetid}/attributes"
GET_ASSETS_ID = "get:assets/{id}"
GET_WORKGROUPS_NAME_ASSETS_NAME = "get:workgroups/{workgroupname}/assets?name"
POST_WORKGROUPS_WORKGROUPID_ASSETS = "post:workgroups/{workgroupid}/assets"

# Smart Rules
GET_SMART_RULES = "get:smartrules"
GET_SMART_RULES_ID = "get:smartrules/{id}"
POST_SMART_RULES_FILTER_ASSET_ATTRIBUTE = "post:smartrules/filterassetattribute"
POST_SMART_RULES_ID_PROCESS = "post:smartrules/{id}/process"

# Organizations
GET_ORGANIZATIONS = "get:organizations"
GET_ORGANIZATIONS_ID = "get:organizations/{id}"
GET_ORGANIZATIONS_NAME = "get:organizations/{name}"

# Workgroups
GET_WORKGROUPS_ID = "get:workgroups/{id}"
GET_WORKGROUPS_NAME = "get:workgroups?name={name}"
GET_WORKGROUPS = "get:workgroups"
POST_WORKGROUPS = "post:workgroups"

# Databases
GET_DATABASES = "get:databases"
GET_DATABASES_ID = "get:databases/{id}"
GET_DATABASES_ASSET_ID = "get:assets/{assetid}/databases"
POST_DATABASES_ASSET_ID = "post:assets/{assetid}/databases"
PUT_DATABASES_ID = "put:databases/{id}"
DELETE_DATABASES_ID = "delete:databases/{id}"

# Address Groups
GET_ADDRESS_GROUPS = "get:addressgroups"
GET_ADDRESS_GROUPS_ID = "get:addressgroups/{id}"
GET_ADDRESS_GROUPS_NAME = "get:addressgroups/{name}"

# Users
GET_USERS = "get:users"
GET_USERS_ID = "get:users/{id}"
GET_USERS_USERGROUPID = "get:usergroups/{usergroupid}/users"
POST_USERS_USERGROUPID = "post:usergroups/{usergroupid}/users"
POST_USERS_BI = "post:users:bi"
POST_USERS_AD = "post:users:ad"
POST_USERS_LDAP = "post:users:ldap"
POST_USERS_APP = "post:users:app"
POST_USERS_QUARANTINE = "post:users/{id}/quarantine"
POST_USERS_RECYCLE_CLIENT_SECRET = "post:users/{id}/recycleclientsecret"  # nosec
PUT_USERS_ID_BI = "put:users/{id}:bi"
PUT_USERS_ID_APP = "put:users/{id}:app"

# Managed Accounts
GET_MANAGED_SYSTEMS_SYSTEM_ID_MANAGED_ACCOUNTS = (
    "get:managedsystems/{systemid}/managedaccounts"
)
GET_MANAGED_ACCOUNTS_ID = "get:managedaccounts/{id}"
POST_MANAGED_SYSTEMS_SYSTEM_ID_MANAGED_ACCOUNTS = (
    "post:managedsystems/{systemid}/managedaccounts"
)
GET_MANAGED_ACCOUNTS_ID_ATTRIBUTES = "get:managedaccounts/{managedaccountid}/attributes"

# Functional Accounts
GET_FUNCTIONAL_ACCOUNTS = "get:functionalaccounts"
GET_FUNCTIONAL_ACCOUNTS_ID = "get:functionalaccounts/{id}"
POST_FUNCTIONAL_ACCOUNTS = "post:functionalaccounts"

# Entitlements
GET_ENTITLEMENTS = "get:entitlements"
GET_ENTITLEMENTS_GROUP_IDS = "get:entitlements:groupids"

# Entity Types
GET_ENTITY_TYPES = "get:entitytypes"

# User Groups
GET_USERGROUPS = "get:usergroups"
GET_USERGROUPS_ID = "get:usergroups/{id}"
GET_USERGROUPS_NAME = "get:usergroups?name={name}"
POST_USERGROUPS_BI = "post:usergroups:bi"
POST_USERGROUPS_ENTRAID = "post:usergroups:entraid"
POST_USERGROUPS_AD = "post:usergroups:ad"
POST_USERGROUPS_LDAP = "post:usergroups:ldap"
DELETE_USERGROUPS = "delete:usergroups?name={name}"
GET_USERGROUPS_ID_SMARTRULES = "get:usergroups/{id}/smartrules"

# Roles
GET_ROLES = "get:roles"

# User group roles constants for GET and POST
GET_USERGROUPS_ID_SMARTRULES_ROLES = (
    "get:usergroups/{id}/smartrules/{smartruleid}/roles"
)
POST_USERGROUPS_ID_SMARTRULES_ROLES = (
    "post:usergroups/{id}/smartrules/{smartruleid}/roles"
)

# ISA Requests
POST_ISA_REQUESTS = "post:isarequests"

# Platforms
GET_PLATFORMS = "get:platforms"

# Operating Systems
GET_OPERATING_SYSTEMS = "get:operatingsystems"

# Requests
GET_REQUESTS = "get:requests"
POST_REQUESTS = "post:requests"
POST_REQUESTS_ALIASES = "post:aliases/{aliasid}/requests"
PUT_REQUESTS_CHECKIN = "put:requests/{requestid}/checkin"
PUT_REQUESTS_APPROVE = "put:requests/{requestid}/approve"
PUT_REQUESTS_DENY = "put:requests/{requestid}/deny"
PUT_REQUESTS_ROTATE_ON_CHECKIN = "put:requests/{requestid}/rotateoncheckin"

# Request Termination
POST_REQUEST_TERMINATION_MANAGED_ACCOUNT_ID = (
    "post:managedaccounts/{managedaccountid}/requests/terminate"
)
POST_REQUEST_TERMINATION_MANAGED_SYSTEM_ID = (
    "post:managedsystems/{managedsystemid}/requests/terminate"
)
POST_REQUEST_TERMINATION_USER_ID = "post:users/{userid}/requests/terminate"

# Request Sets
GET_REQUEST_SETS = "get:requestsets"
POST_REQUEST_SETS = "post:requestsets"

# Password Rules
GET_PASSWORD_RULES = "get:passwordrules"  # nosec
GET_PASSWORD_RULES_ID = "get:passwordrules/{id}"  # nosec
GET_PASSWORD_RULES_ENABLED_PRODUCTS = (
    "get:passwordrules?enabledproducts={productname}"  # nosec
)

# Permissions
GET_PERMISSIONS = "get:permissions"
GET_USERGROUP_PERMISSIONS = "get:usergroups/{usergroupid}/permissions"
POST_USERGROUP_PERMISSIONS = "post:usergroups/{usergroupid}/permissions"
DELETE_USERGROUP_PERMISSIONS = "delete:usergroups/{usergroupid}/permissions"

# Credentials
GET_CREDENTIALS_REQUESTID = "get:credentials/{requestid}"
GET_CREDENTIALS_ALIASID = "get:aliases/{aliasid}/credentials/{requestid}"

# Access Policies
GET_ACCESS_POLICIES = "get:accesspolicies"
POST_ACCESS_POLICIES_TEST = "post:accesspolicies/test"

# API Registrations
GET_API_REGISTRATIONS = "get:apiregistrations"
GET_API_REGISTRATIONS_ID = "get:apiregistrations/{id}"
POST_API_REGISTRATIONS = "post:apiregistrations"
PUT_API_REGISTRATIONS_ID = "put:apiregistrations/{id}"

# Access levels
GET_ACCESS_LEVELS = "get:accesslevels"
POST_ACCESS_LEVELS_USERGROUPID_SMARTRULEID = (
    "post:usergroups/{usergroupid}/smartrules/{smartruleid}/accesslevels"
)

# Sessions
GET_SESSIONS = "get:sessions"
GET_SESSIONS_ID = "get:sessions/{id}"
POST_SESSIONS_REQUEST_ID = "post:requests/{requestid}/sessions"
POST_SESSIONS_ADMIN = "post:sessions/admin"

# Session locking
POST_SESSION_LOCK_SESSIONID = "post:sessions/{sessionid}/lock"
POST_SESSION_LOCK_MANAGED_ACCOUNT_ID = (
    "post:managedaccounts/{managedaccountid}/sessions/lock"
)
POST_SESSION_LOCK_MANAGED_SYSTEM_ID = (
    "post:managedsystems/{managedsystemid}/sessions/lock"
)

# Session termination
POST_SESSION_TERMINATE_SESSIONID = "post:sessions/{sessionid}/terminate"
POST_SESSION_TERMINATE_MANAGED_ACCOUNT_ID = (
    "post:managedaccounts/{managedaccountid}/sessions/terminate"
)
POST_SESSION_TERMINATE_MANAGED_SYSTEM_ID = (
    "post:managedsystems/{managedsystemid}/sessions/terminate"
)

# Quick Rules
GET_QUICK_RULES = "get:quickrules"
GET_QUICK_RULES_ID = "get:quickrules/{id}"

# Keystrokes
GET_SESSIONS_SESSIONID_KEYSTROKES = "get:sessions/{sessionid}/keystrokes"
GET_KEYSTROKES_ID = "get:keystrokes/{id}"
POST_KEYSTROKES_SEARCH = "post:keystrokes/search"

# Subscription Deliveries
GET_SUBSCRIPTIONS_DELIVERY = "get:subscriptions/delivery"
POST_SUBSCRIPTIONS_DELIVERY_DOWNLOAD = "post:subscriptions/delivery/download"

# EPM Policies
POST_EPM_POLICIES_ID_EPMAPPLICATIONS_ADD = "post:epmpolicies/{id}/epmapplications/add"

# Imports
POST_IMPORTS = "post:imports"

# Applications
GET_APPLICATIONS = "get:applications"
GET_APPLICATIONS_ID = "get:applications/{id}"
POST_MANAGED_ACCOUNT_APPLICATIONS = "post:managed_accounts/applications/{applicationid}"
DELETE_MANAGED_ACCOUNT_APPLICATIONS = (
    "delete:managed_accounts/applications/{applicationid}"
)

# Attribute types
GET_ATTRIBUTE_TYPES = "get:attributetypes"
GET_ATTRIBUTE_TYPES_ID = "get:attributetypes/{id}"
POST_ATTRIBUTE_TYPES = "post:attributetypes"
DELETE_ATTRIBUTE_TYPES_ID = "delete:attributetypes/{id}"

# Attributes
GET_ATTRIBUTES_ATTRIBUTE_TYPE_ID = "get:attributetypes/{id}/attributes"
GET_ATTRIBUTE_ID = "get:attributes/{id}"
POST_ATTRIBUTE_ATTRIBUTE_TYPE_ID = "post:attributetypes/{id}/attributes"
DELETE_ATTRIBUTE_ID = "delete:attributes/{id}"
GET_ATTRIBUTES_MANAGED_ACCOUNT_ID = "get:managedaccounts/{managedaccountid}/attributes"
GET_ATTRIBUTES_MANAGED_SYSTEM_ID = "get:managedsystems/{managedsystemid}/attributes"
POST_ATTRIBUTE_MANAGED_ACCOUNT_ID = (
    "post:managedaccounts/{managedaccountid}/attributes/{attributeid}"
)
POST_ATTRIBUTE_MANAGED_SYSTEM_ID = (
    "post:managedsystems/{managedsystemid}/attributes/{attributeid}"
)
DELETE_ATTRIBUTES_MANAGED_ACCOUNT_ID = (
    "delete:managedaccounts/{managedaccountid}/attributes"
)
DELETE_ATTRIBUTE_MANAGED_ACCOUNT_ID = (
    "delete:managedaccounts/{managedaccountid}/attributes/{attributeid}"
)
DELETE_ATTRIBUTES_MANAGED_SYSTEM_ID = (
    "delete:managedsystems/{managedsystemid}/attributes"
)
DELETE_ATTRIBUTE_MANAGED_SYSTEM_ID = (
    "delete:managedsystems/{managedsystemid}/attributes/{attributeid}"
)

# Ticket Systems
GET_TICKET_SYSTEMS = "get:ticketsystems"

# Aliases
GET_ALIASES = "get:aliases"
GET_ALIASES_ID = "get:aliases/{id}"
GET_ALIASES_NAME = "get:aliases?name={name}"

# Propagation action types
GET_PROPAGATION_ACTION_TYPES = "get:propagationactiontypes"

# Propagation actions
GET_PROPAGATION_ACTIONS = "get:propagationactions"
GET_PROPAGATION_ACTIONS_ID = "get:propagationactions/{id}"
GET_MANAGED_ACCOUNTS_PROPAGATION_ACTIONS = (
    "get:managedaccounts/{managedaccountid}/propagationactions"
)
POST_MANAGED_ACCOUNT_PROPAGATION_ACTIONS = (
    "post:managedaccounts/{managedaccountid}/propagationactions/{propagationactionid}"
)
DELETE_MANAGED_ACCOUNTS_PROPAGATION_ACTIONS = (
    "delete:managedaccounts/{managedaccountid}/propagationactions"
)
DELETE_MANAGED_ACCOUNT_ID_PROPAGATION_ACTION_ID = (
    "delete:managedaccounts/{managedaccountid}/propagationactions/{propagationactionid}"
)

# DSS Key policies
GET_DSS_KEY_RULES = "get:dsskeyrules"
GET_DSS_KEY_RULES_ID = "get:dsskeyrules/{id}"



"""Operating Systems module, all the logic to manage operating systems from BI API"""

import logging

from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import ListMixin


class OperatingSystem(APIObject, ListMixin):
    """
    OperatingSystem class for managing operating systems via the PS API.

    This class provides functionality to list operating systems.
    Inherits from APIObject and ListMixin to provide basic API functionality
    and list operations.
    """

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/operatingsystems")



"""
This file contains the fields mapping for each endpoint and version related to
DSS key Policies.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/password-safe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_DSS_KEY_RULES,
    GET_DSS_KEY_RULES_ID,
)
from secrets_safe_library.constants.versions import Version

DEFAULT_FIELDS = [
    "DSSKeyRuleID",
    "Name",
    "Description",
    "KeyType",
    "KeySize",
    "EncryptionType",
    "PasswordRuleID",
]

fields = {
    GET_DSS_KEY_RULES: {
        Version.DEFAULT: DEFAULT_FIELDS,
    },
    GET_DSS_KEY_RULES_ID: {
        Version.DEFAULT: DEFAULT_FIELDS,
    },
}
